var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
import { loadScript } from '../autocapture-utils';
import { _getHashParam, _register_event } from '../utils';
var Toolbar = /** @class */ (function () {
    function Toolbar(instance) {
        this.instance = instance;
    }
    Toolbar.prototype.afterDecideResponse = function (response) {
        var editorParams = response['editorParams'] ||
            (response['toolbarVersion'] ? { toolbarVersion: response['toolbarVersion'] } : {});
        if (response['isAuthenticated'] &&
            editorParams['toolbarVersion'] &&
            editorParams['toolbarVersion'].indexOf('toolbar') === 0) {
            this._loadEditor(__assign(__assign({}, editorParams), { apiURL: this.instance.get_config('api_host') }));
        }
    };
    /**
     * To load the visual editor, we need an access token and other state. That state comes from one of three places:
     * 1. In the URL hash params if the customer is using an old snippet
     * 2. From session storage under the key `editorParams` if the editor was initialized on a previous page
     */
    Toolbar.prototype.maybeLoadEditor = function (location, localStorage, history) {
        if (location === void 0) { location = window.location; }
        if (localStorage === void 0) { localStorage = undefined; }
        if (history === void 0) { history = window.history; }
        try {
            // Before running the code we check if we can access localStorage, if not we opt-out
            if (!localStorage) {
                try {
                    window.localStorage.setItem('test', 'test');
                    window.localStorage.removeItem('test');
                }
                catch (error) {
                    return false;
                }
                // If localStorage was undefined, and localStorage is supported we set the default value
                localStorage = window.localStorage;
            }
            var stateHash = _getHashParam(location.hash, '__posthog') || _getHashParam(location.hash, 'state');
            var state = stateHash ? JSON.parse(decodeURIComponent(stateHash)) : null;
            var parseFromUrl = state && (state['action'] === 'mpeditor' || state['action'] === 'ph_authorize');
            var editorParams = void 0;
            if (parseFromUrl) {
                // happens if they are initializing the editor using an old snippet
                editorParams = state;
                if (editorParams && Object.keys(editorParams).length > 0) {
                    localStorage.setItem('_postHogEditorParams', JSON.stringify(editorParams));
                    if (state['desiredHash']) {
                        // hash that was in the url before the redirect
                        location.hash = state['desiredHash'];
                    }
                    else if (history) {
                        history.replaceState('', document.title, location.pathname + location.search); // completely remove hash
                    }
                    else {
                        location.hash = ''; // clear hash (but leaves # unfortunately)
                    }
                }
            }
            else {
                // get credentials from localStorage from a previous initialzation
                editorParams = JSON.parse(localStorage.getItem('_postHogEditorParams') || '{}');
                // delete "add-action" or other intent from editorParams, otherwise we'll have the same intent
                // every time we open the page (e.g. you just visiting your own site an hour later)
                delete editorParams.userIntent;
            }
            if (!editorParams.apiURL) {
                editorParams.apiURL = this.instance.get_config('api_host');
            }
            if (editorParams['token'] && this.instance.get_config('token') === editorParams['token']) {
                this._loadEditor(editorParams);
                return true;
            }
            else {
                return false;
            }
        }
        catch (e) {
            return false;
        }
    };
    Toolbar.prototype._loadEditor = function (editorParams) {
        var _this = this;
        if (!window['_postHogToolbarLoaded']) {
            // only load the codeless event editor once, even if there are multiple instances of PostHogLib
            ;
            window['_postHogToolbarLoaded'] = true;
            var host = editorParams['jsURL'] || editorParams['apiURL'] || this.instance.get_config('api_host');
            var toolbarScript = 'toolbar.js';
            var editorUrl = host + (host.endsWith('/') ? '' : '/') + 'static/' + toolbarScript + '?_ts=' + new Date().getTime();
            var disableToolbarMetrics = this.instance.get_config('api_host') !== 'https://app.posthog.com' &&
                this.instance.get_config('advanced_disable_toolbar_metrics');
            editorParams = __assign(__assign({}, editorParams), (disableToolbarMetrics ? { instrument: false } : {}));
            loadScript(editorUrl, function () {
                ;
                window['ph_load_editor'](editorParams, _this.instance);
            });
            // Turbolinks doesn't fire an onload event but does replace the entire page, including the toolbar
            _register_event(window, 'turbolinks:load', function () {
                ;
                window['_postHogToolbarLoaded'] = false;
                _this._loadEditor(editorParams);
            });
            return true;
        }
        return false;
    };
    return Toolbar;
}());
export { Toolbar };
//# sourceMappingURL=toolbar.js.map